import bb.cascades 1.3
import bb.cascades.multimedia 1.0
import bb.multimedia 1.0
import bb.system 1.2

Page {
    property string scannedData: ""
    property bool macroMode: false

    // Handles the data recognized by the barcode detector
    function handleDetection(data) {
        // We consider only the first detection and ignore the consequent ones
        if (scannedData.length > 0) 
            return;
        
        scannedData = data;
        var url = _app.normalizeUrl(data); // trying to make it a URL
        if (url.length > 0) {
            scannedData = url;
            console.log("invoke: " + url);
            if (!_app.tryInvokeUrl(url)) {
                // Could not invoke the URL, so just show it
                resultToast.show();
            }
        } else {
            // not a link - show as text
            resultToast.show();
        }
    }

    function quit() {
        camera.close();
        // Minimize + quit looks better than just quit
        Application.minimize();
        Application.quit();
    }
     
    onCreationCompleted: {
        // Free the camera when the app is minimized/closed
        Application.asleep.connect(camera.close);
        Application.awake.connect(camera.open);
        Application.aboutToQuit.connect(camera.close);
        _app.invocationFailed.connect(function(url) {
            // Don't show the error, simply show the URL as text
            resultToast.show();
        });
        console.log("Started");
    }

    onMacroModeChanged: {
        camera.getSettings(cameraSettings);
        if (macroMode) {
            cameraSettings.focusMode = CameraFocusMode.ContinuousMacro;
        } else {
            cameraSettings.focusMode = CameraFocusMode.ContinuousAuto;
        }
        camera.applySettings(cameraSettings);
    }
    
    actions: [
        ActionItem {
            id: macroAction
            imageSource: "asset:///images/ic_macro.png"
            title: qsTr("Macro", "A button/action which toggles camera macro (close-up) mode") + Retranslate.onLocaleOrLanguageChanged
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                macroMode = !macroMode;
            }
        },
        ActionItem {
            id: lightAction
            imageSource: "asset:///images/ic_light.png"
            title: qsTr("Light", "A button/action which toggles phone's flashlight (in continuous mode)") + Retranslate.onLocaleOrLanguageChanged
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                camera.capturePhoto(); // turns on the flash
            }
        },
        ActionItem {
            id: reviewAction
            imageSource: "asset:///images/ic_rate_app.png"
            title: qsTr("Review", "A button/action to write a review about this app in app store") + Retranslate.onLocaleOrLanguageChanged
            ActionBar.placement: _app.reviewDone ? ActionBarPlacement.InOverflow : ActionBarPlacement.OnBar
            onTriggered: {
                _app.reviewDone = true; // well, at least so we hope
                Qt.openUrlExternally("appworld://content/59960946");
            }
        }
    ]
    
    Container {
        onTouchCapture: {
            if (scannedData.length > 0) {
                // User tapped away from the Copy button
                resultToast.cancel();
                quit();
            }
        }

        layout: DockLayout {}
        background: Color.Black
        // Camera preview
        Camera {
            id: camera

            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill

            onCameraOpened: {
                console.log("camera opened");
                
                // Apply some settings after the camera was opened successfully
                getSettings(cameraSettings);
                cameraSettings.cameraMode = CameraMode.Photo;
                cameraSettings.focusMode = CameraFocusMode.ContinuousAuto;
                cameraSettings.shootingMode = CameraShootingMode.Stabilization;
                cameraSettings.cameraRollPath = "/dev/null"; // we only need flash, not the photos
                cameraSettings.flashMode = CameraFlashMode.On;
                applySettings(cameraSettings);
                camera.startViewfinder();
            }
            onViewfinderStarted: {
                console.log("viewfinderStarted");
            }
            onViewfinderStartFailed: {
                console.log("viewfinderStartFailed with code " + error);
            }
            onCameraClosed: {
                console.log("camera closed");
            }

            attachedObjects: [
                BarcodeDetector {
                    id: barcodeDetector
                    camera: camera
                    formats: BarcodeFormat.Any
                    onDetected: handleDetection(data)
                },
                CameraSettings {
                    id: cameraSettings
                },
                SystemToast {
                    id: invokeFailToast
                    position: SystemUiPosition.BottomCenter
                    body: qsTr("Failed to open the URL", "An error message when the app could not open the link in a system browser.")
                },
                SystemToast {
                    id: resultToast
                    position: SystemUiPosition.MiddleCenter
                    body: scannedData
                    button.label: qsTr("Copy", "A button/action which copies text to clipboard. Keep it as short as possible.")
                    onFinished: {
                        if (value == SystemUiResult.ButtonSelection) {
                            _app.copyToClipboard(scannedData);
                        }
                        quit();
                    }
                }
            ]
        }
        ImageView {
            horizontalAlignment: HorizontalAlignment.Right
            verticalAlignment: VerticalAlignment.Top
            imageSource: "asset:///images/macro_overlay.png"
            visible: macroMode
        }
    }
    
    Menu.definition: MenuDefinition {
        helpAction: HelpActionItem {
            title: qsTr("About", "A button/action which shows information about the app.") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "asset:///images/ic_about.png"
            onTriggered: {
                aboutSheet.open();
            }
            attachedObjects: [
                AboutSheet {
                    id: aboutSheet
                }
            ]
        }
    }
}
